{
    Файл является частью библиотеки RubiRobot

    © 2017 Слинкин Д.А.

    Обработка двигателей контроллера Lego EV3

    Подробная информация о лицензировании находится в файле README,
    входящем в состав библиотеки.

    ДАННОЕ ПРОГРАММНОЕ ОБЕСПЕЧЕНИЕ ПРЕДОСТАВЛЯЕТСЯ «КАК ЕСТЬ», БЕЗ
    КАКИХ-ЛИБО ГАРАНТИЙ, ЯВНО ВЫРАЖЕННЫХ ИЛИ ПОДРАЗУМЕВАЕМЫХ,
    ВКЛЮЧАЯ ГАРАНТИИ ТОВАРНОЙ ПРИГОДНОСТИ, СООТВЕТСТВИЯ ПО ЕГО
    КОНКРЕТНОМУ НАЗНАЧЕНИЮ И ОТСУТСТВИЯ НАРУШЕНИЙ, НО НЕ ОГРАНИЧИВАЯСЬ 
    ИМИ. НИ В КАКОМ СЛУЧАЕ АВТОРЫ ИЛИ ПРАВООБЛАДАТЕЛИ НЕ НЕСУТ
    ОТВЕТСТВЕННОСТИ ПО КАКИМ-ЛИБО ИСКАМ, ЗА УЩЕРБ ИЛИ ПО ИНЫМ ТРЕБОВАНИЯМ,
    В ТОМ ЧИСЛЕ, ПРИ ДЕЙСТВИИ КОНТРАКТА, ДЕЛИКТЕ ИЛИ ИНОЙ СИТУАЦИИ,
    ВОЗНИКШИМ ИЗ-ЗА ИСПОЛЬЗОВАНИЯ ПРОГРАММНОГО ОБЕСПЕЧЕНИЯ ИЛИ ИНЫХ
    ДЕЙСТВИЙ С ПРОГРАММНЫМ ОБЕСПЕЧЕНИЕМ.

 **********************************************************************}


{$SMARTLINK ON}
{$mode objfpc}
{$H+}

unit rubiroMotors;


interface
uses BaseUnix,uev3Devices,uev3sysfs;
{

    run-forever: Causes the motor to run until another command is sent.
    run-to-abs-pos: Runs the motor to an absolute position specified by``position_sp`` and then stops the motor using the command specified in stop_action.
    run-to-rel-pos: Runs the motor to a position relative to the current position value. The new position will be current position + position_sp. When the new position is reached, the motor will stop using the command specified by stop_action.
    run-timed: Run the motor for the amount of time specified in time_sp and then stops the motor using the command specified by stop_action.
    run-direct: Runs the motor using the duty cycle specified by duty_cycle_sp. Unlike other run commands, changing duty_cycle_sp while running will take effect immediately.
    stop: Stop any of the run commands before they are complete using the command specified by stop_action.
    reset: Resets all of the motor parameter attributes to their default values. This will also have the effect of stopping the motor.
}

{
    running: Power is being sent to the motor.
    ramping: The motor is ramping up or down and has not yet reached a constant output level.
    holding: The motor is not turning, but rather attempting to hold a fixed position.
    overloaded: The motor is turning as fast as possible, but cannot reach its speed_sp.
    stalled: The motor is trying to run but is not turning at all.
}

type  TEv3mCommand=(cRunForever,cRunToAbsPos,cRunToRelPos,cRunTimed,cRunDirect,cStop,cReset);
const ev3mCommands:array[TEv3mCommand] of string[20]=('run-forever','run-to-abs-pos','run-to-rel-pos','run-timed','run-direct','stop','reset');

type  TEv3mStopAction=(saCoast,saBrake,saHold);
const ev3mStopActions:array[TEv3mStopAction] of string[20]=('coast', 'brake', 'hold');

type  TEv3mPolarity=(pNormal,pInversed);
const ev3mPolarities:array[TEv3mPolarity] of string[20]=('normal', 'inversed');

type  TEv3mState=(sStoped,sRunning,sRamping,sHolding,sOverloaded,sStalled);
      TEv3mStates=set of TEv3mState;
const ev3mStates:array[TEv3mState]of string[20]=('','running','ramping','holding','overloaded','stalled');



type

  { TEv3Motor }

  TEv3Motor=class(TEv3OneDeviceHelper)
   private
    procedure resetWriteInterval;
   public
   constructor create(device:variant; port:variant);
   constructor create(device:variant);
   procedure setIntervals(R,W:integer);override;

  end;

  { TEv3TachoMotor }

  TEv3TachoMotor=class(TEv3Motor)
    private
     fmaxSpeed:cint;
     _fspeed:cint;
     fstopaction:TEv3mStopAction;
     fpolarity:TEv3mPolarity;
     fcount_per_rot:cint;
     function getcount_per_rot: integer;
     function getMaxSpeed: cint;
     procedure setFSpeed(AValue: cint);
    protected
     procedure deviceRefreshed(dev:TEv3Device);override;
     property maxSpeed:cint read getMaxSpeed;
     procedure reversePolarity;
     property fspeed:cint read _fspeed write setFSpeed;
    public
     class function varToSA(sa:variant):TEv3mStopAction;
     class function varToState(s:variant):TEv3mStates;
   //  function wait()
     // ждет события из списка событий: sStoped,sRunning,sRamping,sHolding,sOverloaded,sStalled
     function waitEvent(event:ansistring; timeout:dword=high(dword)):boolean; override;
    public
     procedure setIntervals(R,W:integer);override;

     property count_per_rot:integer read getcount_per_rot;
     function wait():TEv3TachoMotor; // ждет остановки мотора

     {
      Устанавливает способ остановки мотора.
      Действует при запуске мотора.
      На текущее вращение мотора влияния не оказывает.
      Варианты:
      1) Остановка с удержанием позиции, минимальный тормозной путь
         saBrake,'brake','тормоз','тормози','тормозить',1
      2) Остановка без отключения подачи электроэнергии, средний тормозной путь
         saHold,'hold','стоп','стоять','стой',2
      0) Движение накатом, с отключением подачи электроэнергии, максимальный тормозной путь
         saCoast,0,'накат' и другие варианты, не входящие в 1) и 2)
     }
     function setHowStop(sa:variant):TEv3TachoMotor;
     {
      Возвращает способ остановки мотора
       1 - Остановка с удержанием позиции, минимальный тормозной путь
       2 - Остановка без отключения подачи электроэнергии, средний тормозной путь
       0 - Движение накатом, с отключением подачи электроэнергии, максимальный тормозной путь
     }
     function howStop(out sa:variant):TEv3TachoMotor;
     function howStop:variant;

     {
       Устанавливает и возвращает текущую позицию мотора
     }
     function setPosition(p:variant):TEv3TachoMotor;
     function position(out p:variant):TEv3TachoMotor;
     function position:variant;

     {
       Устанавливает скорость вращения от -100% до 100%
       Действует при запуске мотора.
       На текущее вращение мотора влияния не оказывает.
     }
     function setSpeed(sp:variant):TEv3TachoMotor;
     {
       Возвращает установленную ранее скорость вращения от -100 до 100
     }
     function Speed(out sp:variant):TEv3TachoMotor;
     function Speed:variant;

     {
       Возвращает реальную скорость вращения от 0 до 100
     }
     function currentSpeed(out sp:variant):TEv3TachoMotor;
     function currentSpeed:variant;

     {
       Устанавливает реверс
       Действует при запуске мотора.
       На текущее вращение мотора влияния не оказывает.
     }
     function setReverse(yes:variant):TEv3TachoMotor;
     {
       Возвращает 1, если установлен реверс и 0 в противном случае
     }
     function Reverse(out yes:variant):TEv3TachoMotor;
     function Reverse:variant;

     {
       Останавливает мотор, способ остановки - см. setHowStop
     }
     function Stop(sa:variant):TEv3TachoMotor;
     function Stop():TEv3TachoMotor;

     {
       Запускает мотор (aсинхронный вызов). Возможные параметры - скорость и способ остановки
       по умолчанию при вызове stop()
     }
     function Run(sp:variant; sa:variant):TEv3TachoMotor;
     function Run(sp:variant):TEv3TachoMotor;
     function Run():TEv3TachoMotor;

     {
       Запускает мотор на количество миллисекунд (aсинхронный вызов).
       Возможные параметры - скорость и способ остановки
     }
     function RunTime(ms:variant; sp:variant; sa:variant):TEv3TachoMotor;
     function RunTime(ms:variant; sp:variant):TEv3TachoMotor;
     function RunTime(ms:variant):TEv3TachoMotor;

     {
       Запускает мотор на количество градусов (aсинхронный вызов).
       Возможные параметры - скорость и способ остановки
     }
     function RunDeg(deg:variant; sp:variant; sa:variant):TEv3TachoMotor;
     function RunDeg(deg:variant; sp:variant):TEv3TachoMotor;
     function RunDeg(deg:variant):TEv3TachoMotor;

     {
       Запускает мотор на количество оборотов (aсинхронный вызов).
       Возможные параметры - скорость и способ остановки
     }
     function RunTurn(turn:variant; sp:variant; sa:variant):TEv3TachoMotor;
     function RunTurn(turn:variant; sp:variant):TEv3TachoMotor;
     function RunTurn(turn:variant):TEv3TachoMotor;

  end;

  { TEv3MediumMotor }

  TEv3MediumMotor=class(TEv3TachoMotor)
   constructor create(port:variant);
   constructor create();
  end;


  { TEv3LargeMotor }

  TEv3LargeMotor=class(TEv3TachoMotor)
   constructor create(port:variant);
   constructor create();
  end;

  { TEv3Rule }

  TEv3Rule=class
   private
    _fspeed:cint;
    fdirect:cint;
    procedure setFSpeed(AValue: cint);
   protected
    property fspeed:cint read _fspeed write setFSpeed;
   public
    L,R:TEv3LargeMotor;
   public
    procedure setIntervals(RR,WW:integer);

    // ждет остановки моторов
    function wait():TEv3Rule;

    constructor create(LPort,RPort:variant);
    constructor create();
    destructor destroy;override;


    // Устанавливает способ остановки моторов (см.TEv3TachoMotor)
    function setHowStop(sa:variant):TEv3Rule;
    // Возвращает способ остановки моторов (см.TEv3TachoMotor)
    function howStop(out sa:variant):TEv3Rule;
    function howStop:variant;

    {
      Устанавливает скорость вращения от -100% до 100% (см.TEv3TachoMotor)
    }
    function setSpeed(sp:variant):TEv3Rule;
    {
      Возвращает установленную ранее скорость вращения от -100 до 100
      (см.TEv3TachoMotor)
    }
    function Speed(out sp:variant):TEv3Rule;
    function Speed:variant;

    {
      Возвращает установленную ранее скорость вращения для левого колеса
    }
    function SpeedLeft(out sp:variant):TEv3Rule;
    function SpeedLeft:variant;

    {
      Возвращает установленную ранее скорость вращения для правого колеса
    }
    function SpeedRight(out sp:variant):TEv3Rule;
    function SpeedRight:variant;

    // устанавливает и возвращает реверс (см.TEv3TachoMotor)
    function setReverse(yes:variant):TEv3Rule;
    function Reverse(out yes:variant):TEv3Rule;
    function Reverse:variant;

    // останавливает оба мотора (см.TEv3TachoMotor)
    function Stop(sa:variant):TEv3Rule;
    function Stop():TEv3Rule;

    {
     запускает оба мотора (aсинхронный вызов) (см.TEv3TachoMotor)
     direct - направление движения
     Примеры поведения рулевого управления при положительной скорости
     и значения direct:
      100 - поворот вправо вокруг оси,
      -100 - поворот влево вокруг оси,
      50 - поворот вправо вокруг правого колеса,
      -50 - поворот влево вокруг левого колеса
    }
    function Run(direct:variant; sp:variant; sa:variant):TEv3Rule;
    function Run(direct:variant; sp:variant):TEv3Rule;
    function Run(direct:variant):TEv3Rule;

    {
      Запускает моторы на количество миллисекунд (aсинхронный вызов).
      Возможные параметры - скорость и способ остановки
    }
    function RunTime(direct:variant; ms:variant; sp:variant; sa:variant):TEv3Rule;
    function RunTime(direct:variant; ms:variant; sp:variant):TEv3Rule;
    function RunTime(direct:variant; ms:variant):TEv3Rule;

    {
      Запускает моторы на количество градусов (БЛОКИРУЮЩИЙ вызов).
      Возможные параметры - скорость и способ остановки
    }
    function RunDegWait(direct:variant; deg:variant; sp:variant; sa:variant):TEv3Rule;
    function RunDegWait(direct:variant; deg:variant; sp:variant):TEv3Rule;
    function RunDegWait(direct:variant; deg:variant):TEv3Rule;

    {
      Запускает моторы на количество оборотов (БЛОКИРУЮЩИЙ вызов).
      Возможные параметры - скорость и способ остановки
    }
    function RunTurnWait(direct:variant; turn:variant; sp:variant; sa:variant):TEv3Rule;
    function RunTurnWait(direct:variant; turn:variant; sp:variant):TEv3Rule;
    function RunTurnWait(direct:variant; turn:variant):TEv3Rule;
  end;


implementation
uses
  uev3,SysUtils,errors,Unix,uev3utf8,variants,dateutils;

{ TEv3Motor }

procedure TEv3Motor.resetWriteInterval;
 var i:TEv3DeviceAttribute;
begin
  for i in TEv3DeviceAttribute do
   if assigned(ev.Attrs[i]) then ev.Attrs[i].skipInterval:=true;
end;

constructor TEv3Motor.create(device: variant; port: variant);
begin
  inherited Create(device, port);
  resetWriteInterval;
end;

constructor TEv3Motor.create(device: variant);
begin
 inherited Create(device);
 resetWriteInterval;
end;

procedure TEv3Motor.setIntervals(R, W: integer);
begin
  // -----
end;


{ TEv3Rule }

constructor TEv3Rule.create(LPort, RPort: variant);
begin
  inherited create;
  L:=TEv3LargeMotor.create(LPort);
  if not assigned(L) then fail;
  R:=TEv3LargeMotor.create(RPort);
  if not assigned(R) then begin L.free; fail; end;
end;

constructor TEv3Rule.create;
begin
  inherited create;
  L:=TEv3LargeMotor.create();
  if not assigned(L) then fail;
  R:=TEv3LargeMotor.create();
  if not assigned(R) then begin L.free; fail; end;
end;

destructor TEv3Rule.destroy;
begin
  l.free;
  r.free;
  inherited destroy;
end;

function TEv3Rule.setHowStop(sa: variant): TEv3Rule;
begin
  result:=self;
  L.setHowStop(sa); R.setHowStop(sa);
end;

function TEv3Rule.howStop(out sa: variant): TEv3Rule;
begin
 result:=self;
 L.HowStop(sa);
end;

function TEv3Rule.howStop: variant;
begin
 result:=L.howStop;
end;

function TEv3Rule.setSpeed(sp: variant): TEv3Rule;
begin
//  writeln('!!! + ',sp,' ',fdirect);
  result:=self;
 fspeed:=sp;
 if fdirect<-100 then fdirect:=-100;
 if fdirect>100 then fdirect:=100;
 if fdirect=0 then begin
  // writeln('!!! ',sp);
   L.setSpeed(sp); R.setSpeed(sp);
  // writeln('!!! ',l.fspeed,' ',r.fspeed);

 end else
 if fdirect<0 then begin
   //writeln('!-! ',sp+(fdirect*sp/50),' ',sp);
   L.setSpeed(sp+(fdirect*double(sp)/50));
   //writeln('------');
   R.setSpeed(sp);
   //writeln('---+++---');
//   writeln(l.speed,' ',r.speed);
 end
 else begin
 // writeln('!+! ',sp,' ',sp+(-fdirect*double(sp)/50));
   L.setSpeed(sp);
   R.setSpeed(sp+(-fdirect*double(sp)/50));
//   writeln(l.speed,' ',r.speed);
 end;
end;

function TEv3Rule.Speed(out sp: variant): TEv3Rule;
begin
 result:=self;
 sp:=self.Speed;
end;

function TEv3Rule.Speed: variant;
begin
  result:=fspeed;
end;

function TEv3Rule.SpeedLeft(out sp: variant): TEv3Rule;
begin
 result:=self;
 L.Speed(sp);
end;

function TEv3Rule.SpeedLeft: variant;
begin
 result:=L.speed;
end;

function TEv3Rule.SpeedRight(out sp: variant): TEv3Rule;
begin
 result:=self;
 R.Speed(sp);
end;

function TEv3Rule.SpeedRight: variant;
begin
 result:=R.speed;
end;


function TEv3Rule.setReverse(yes: variant): TEv3Rule;
begin
 result:=self;
 L.setReverse(yes);
 R.setReverse(yes);
end;

function TEv3Rule.Reverse(out yes: variant): TEv3Rule;
begin
 result:=self;
 L.Reverse(yes);
end;

function TEv3Rule.Reverse: variant;
begin
 result:=L.Reverse;
end;

procedure TEv3Rule.setFSpeed(AValue: cint);
begin
  if avalue>100 then avalue:=100
  else
  if avalue<-100 then avalue:=-100;
  if _fspeed=AValue then Exit;
  _fspeed:=AValue;
end;

procedure TEv3Rule.setIntervals(RR, WW: integer);
begin
  L.setIntervals(RR,WW);
  R.setIntervals(RR,WW);
end;

function TEv3Rule.wait: TEv3Rule;
begin
  //sleep(300);
  result:=self;
  if fdirect>0 then begin
   R.Wait;
   L.Stop;
  // writeln('end');
  end
  else begin
   L.Wait;
   R.Stop;
  end;
  //waitAnyEvent([self,sStoped,self,sHolding,self,sStalled]);
  //sStoped,sRunning,sRamping,sHolding,sOverloaded,sStalled
end;


function TEv3Rule.Stop(sa: variant): TEv3Rule;
begin
 result:=self;
 L.Stop(sa);
 R.Stop(sa);
end;

function TEv3Rule.Stop: TEv3Rule;
begin
 result:=self;
 L.Stop();
 R.Stop();
end;

function TEv3Rule.Run(direct: variant; sp: variant; sa: variant): TEv3Rule;
begin
  setHowStop(sa);
  result:=self.run(direct,sp);
end;

function TEv3Rule.Run(direct: variant; sp: variant): TEv3Rule;
begin
 result:=self;
 fdirect:=direct;
 setSpeed(sp);
 L.Run;
 R.Run;
end;

function TEv3Rule.Run(direct: variant): TEv3Rule;
begin
 result:=self.Run(direct,speed);
end;

function TEv3Rule.RunTime(direct: variant; ms: variant; sp: variant; sa: variant
  ): TEv3Rule;
begin
 setHowStop(sa);
 result:=self.RunTime(direct,ms,sp);
end;

function TEv3Rule.RunTime(direct: variant; ms: variant; sp: variant): TEv3Rule;
begin
 result:=self;
 fdirect:=direct;
 setSpeed(sp);
 L.RunTime(ms);
 R.RunTime(ms);
end;

function TEv3Rule.RunTime(direct: variant; ms: variant): TEv3Rule;
begin
 result:=self.RunTime(direct,ms,speed);
end;

function TEv3Rule.RunDegWait(direct: variant; deg: variant; sp: variant; sa: variant
  ): TEv3Rule;
begin
 setHowStop(sa);
 result:=self.RunDegWait(direct,deg,sp);
end;

function TEv3Rule.RunDegWait(direct: variant; deg: variant; sp: variant): TEv3Rule;
begin
 result:=RunTurnWait(direct,deg/360.0,sp);
end;

function TEv3Rule.RunDegWait(direct: variant; deg: variant): TEv3Rule;
begin
 result:=self.RunDegWait(direct,deg,speed);
end;

function TEv3Rule.RunTurnWait(direct: variant; turn: variant; sp: variant;
  sa: variant): TEv3Rule;
begin
 setHowStop(sa);
 result:=self.RunTurnWait(direct,turn,sp);
end;

function TEv3Rule.RunTurnWait(direct: variant; turn: variant; sp: variant
  ): TEv3Rule;
begin
 result:=self;
 fdirect:=direct;
 setSpeed(sp);
 if fdirect>=0 then begin
//  writeln(r.fspeed,' ',l.fspeed);
  if turn<0 then R.setFSpeed(-R.fspeed);
  R.Run;
  L.RunTurn(turn).wait();
  R.Stop;
 end else begin
  //if turn>=0 then L.RunTurn(high(smallint)) else L.RunTurn(low(smallint)) ;
  if turn<0 then L.setFSpeed(-L.fspeed);
  L.Run;
  R.RunTurn(turn).wait();
  L.Stop;
 end;

end;

function TEv3Rule.RunTurnWait(direct: variant; turn: variant): TEv3Rule;
begin
 result:=self;
 self.RunTurnWait(direct,Turn,speed);
end;

{ TEv3LargeMotor }

constructor TEv3LargeMotor.create(port: variant);
begin
 inherited create(ev3tLTachoMotor,varToPort(port));
end;

constructor TEv3LargeMotor.create;
begin
 inherited create(ev3tLTachoMotor,ev3pUnknown);
end;

{ TEv3MediumMotor }

constructor TEv3MediumMotor.create(port: variant);
begin
 inherited create(ev3tMTachoMotor,varToPort(port));
end;

constructor TEv3MediumMotor.create;
begin
 inherited create(ev3tMTachoMotor,ev3pUnknown);
end;


{ TEv3TachoMotor }

function TEv3TachoMotor.getMaxSpeed: cint;
begin
  if fmaxspeed=0 then fmaxspeed:=ev.Attrs[ev3amax_speed].iData;
  result:=fmaxspeed;
end;

function TEv3TachoMotor.getcount_per_rot: integer;
begin
  if fcount_per_rot=0 then fcount_per_rot:=ev.Attrs[ev3acount_per_rot].iData;
  exit(fcount_per_rot);
end;

procedure TEv3TachoMotor.setFSpeed(AValue: cint);
begin
  if avalue>100 then avalue:=100
  else
  if avalue<-100 then avalue:=-100;
  if _fspeed=AValue then Exit;
  _fspeed:=AValue;
end;

procedure TEv3TachoMotor.deviceRefreshed(dev: TEv3Device);
begin
  inherited deviceRefreshed(dev);
  dev.Attrs[ev3acommand].sData:=ev3mCommands[cReset];
  dev.Attrs[ev3astop_action].sData:=ev3mStopActions[fstopaction];
  dev.Attrs[ev3apolarity].sData:=ev3mPolarities[fpolarity];
end;

procedure TEv3TachoMotor.reversePolarity;
begin
  if reverse then setReverse(0) else setReverse(1);
end;

class function TEv3TachoMotor.varToSA(sa: variant): TEv3mStopAction;
begin
 case utf8lowercase(trim(varToStr(sa))) of
  'brake','тормоз','тормози','тормозить','1':result:=saBrake;
  'hold','стоп','стоять','стой','2':result:=saHold
  else result:=saCoast;
 end;
end;


procedure outSet(s:TEv3mStates);
 var i:TEv3mState;
begin
 for i in s do write(i,' ');
 writeln;
end;

class function TEv3TachoMotor.varToState(s: variant): TEv3mStates;
 var us:string;
     st:TEv3mState;
begin
 us:=utf8lowercase(trim(varToStr(s)));
 result:=[];
 // sStoped,sRunning,sRamping,sHolding,sOverloaded,sStalled
 for st in TEv3mState do begin
  // writeln(us,' == ',ev3mStates[st],' +- ', pos(ev3mStates[st],us)<>0 );
   if (pos(ev3mStates[st],us)<>0)
   //(ev3mStates[st]=us)
   or(us=inttostr(ord(st))) then include(result,st);
 end;
 if result<>[] then exit;
 case us of
  'работа':include(result,sRunning);
  'разгон':include(result,sRamping);
  'стоп':include(result,sHolding);
  'перегрузка':include(result,sOverloaded);
  'блок':include(result,sStalled)
  else include(result,sStoped);
 end;
end;


function TEv3TachoMotor.wait: TEv3TachoMotor;
begin
  result:=self;
//  sleep(100);
 //writeln('beg');
  waitAnyEvent([self,sStoped,self,sHolding,self,sStalled]);
 // writeln('end');
//  waitAnyEvent([self,sStoped]);//,self,sHolding,self,sStalled]);
  //sStoped,sRunning,sRamping,sHolding,sOverloaded,sStalled
end;



function TEv3TachoMotor.waitEvent(event: ansistring; timeout: dword): boolean;
var //res:boolean;
    dt:tdatetime;
    st:TEv3mStates;
    s:string;
begin
// writeln(12321) ;
 st:=varToState(event);
 dt:=now();
 while true do begin
 // writeln(ev.Attrs[ev3astate].sData);
  s:=ev.Attrs[ev3astate].sData;
//  writeln('+++ ',s);
//  write('++++ '); outset(varToState(s));
//  write('+++++ '); outset(st);
  if varToState(s)*st<>[] then begin sleep(10);//write('!!!!! ',ev.Attrs[ev3aposition_sp].sData); outSet(varToState(s)*st);
    exit(true);end;
  if MilliSecondsBetween(now(),dt)>=timeout then exit(false);
  sleep(waitInterval);
 end;
end;

procedure TEv3TachoMotor.setIntervals(R, W: integer);
begin
 ev.Attrs[ev3astop_action].setIntervals(R,W);
 ev.Attrs[ev3aposition].setIntervals(R,W);
 ev.Attrs[ev3aspeed_sp].setIntervals(R,W);
 ev.Attrs[ev3aspeed].setIntervals(R,W);
 ev.Attrs[ev3apolarity].setIntervals(R,W);
 ev.Attrs[ev3acommand].setIntervals(R,W);
 ev.Attrs[ev3atime_sp].setIntervals(R,W);
 ev.Attrs[ev3aposition_sp].setIntervals(R,W);
end;

function TEv3TachoMotor.setHowStop(sa: variant): TEv3TachoMotor;
begin
  result:=self;
  fstopaction:=varToSA(sa);
  ev.Attrs[ev3astop_action].sData:=ev3mStopActions[fStopAction];
end;

function TEv3TachoMotor.howStop(out sa: variant): TEv3TachoMotor;
begin
 result:=self;
 sa:=self.howStop;
end;

function TEv3TachoMotor.howStop: variant;
begin
 result:=varToSA(ev.Attrs[ev3astop_action].sData);
end;

function TEv3TachoMotor.setPosition(p: variant): TEv3TachoMotor;
begin
 result:=self;
 ev.Attrs[ev3aposition].sData:=p;
end;

function TEv3TachoMotor.position(out p: variant): TEv3TachoMotor;
begin
 result:=self;
 p:=ev.Attrs[ev3aposition].iData;
end;

function TEv3TachoMotor.position: variant;
begin
 result:=ev.Attrs[ev3aposition].iData;
end;

function TEv3TachoMotor.setSpeed(sp: variant): TEv3TachoMotor;
begin
 result:=self;
 fspeed:=sp;
 if sp<0 then setReverse(1) else setReverse(0);
 ev.Attrs[ev3aspeed_sp].iData:=(fspeed*maxSpeed) div 100;
end;

function TEv3TachoMotor.Speed(out sp: variant): TEv3TachoMotor;
begin
  result:=self;
  sp:=self.speed;
end;

function TEv3TachoMotor.Speed: variant;
begin
 result:=fspeed;
end;

function TEv3TachoMotor.currentSpeed(out sp: variant): TEv3TachoMotor;
begin
 result:=self;
 sp:=self.currentSpeed;
end;

function TEv3TachoMotor.currentSpeed: variant;
begin
 result:=(ev.Attrs[ev3aspeed].iData*100) div maxSpeed;
end;

function TEv3TachoMotor.setReverse(yes: variant): TEv3TachoMotor;
begin
 result:=self;
// writeln('Reverse = ',TEv3mPolarity(varToYes(yes)));
 ev.Attrs[ev3apolarity].sData:=ev3mPolarities[TEv3mPolarity(varToYes(yes))];
end;

function TEv3TachoMotor.Reverse(out yes: variant): TEv3TachoMotor;
begin
 result:=self;
 yes:=self.reverse;
end;

function TEv3TachoMotor.Reverse: variant;
begin
 result:=ev3mPolarities[pInversed]=ev.Attrs[ev3apolarity].sData;
end;

function TEv3TachoMotor.Stop(sa: variant): TEv3TachoMotor;
begin
 result:=self;
 setHowStop(sa);
 self.stop;
end;

function TEv3TachoMotor.Stop: TEv3TachoMotor;
begin
  result:=self;
  ev.Attrs[ev3astop_action].sData:=ev3mStopActions[fstopaction];
  ev.Attrs[ev3aposition_sp].iData:=0;
  ev.Attrs[ev3acommand].sData:=ev3mCommands[cRunToRelPos];
  // команда собственно остановки не нужна,
  // т.к. в перемещение в относит. позицию 0 уже сделано,
  // это косвенно дает остановку
  // ev.Attrs[ev3acommand].sData:=ev3mCommands[cStop];
end;


function TEv3TachoMotor.Run(sp: variant; sa: variant): TEv3TachoMotor;
begin
  setHowStop(sa);
  result:=self.Run(sp);
end;

function TEv3TachoMotor.Run(sp: variant): TEv3TachoMotor;

begin
 result:=self;
{
 setReverse(0);
 fspeed:=sp;
 ev.Attrs[ev3aduty_cycle_sp].sData:=sp;
 ev.Attrs[ev3acommand].sData:=ev3mCommands[cRunDirect];
}
//setSpeed(sp);
fspeed:=sp;
setReverse(0);
//if sp<0 then setReverse(1) else setReverse(0);
ev.Attrs[ev3aspeed_sp].iData:=((fspeed*maxSpeed) div 100);
ev.Attrs[ev3acommand].sData:=ev3mCommands[cRunForever];
end;

function TEv3TachoMotor.Run: TEv3TachoMotor;
begin
 result:=self;
 self.Run(fspeed);
end;

function TEv3TachoMotor.RunTime(ms: variant; sp: variant; sa: variant
  ): TEv3TachoMotor;
begin
  setHowStop(sa);
  result:=self.runTime(ms,sp);
end;

function TEv3TachoMotor.RunTime(ms: variant; sp: variant): TEv3TachoMotor;
begin
 result:=self;
 setSpeed(sp);
 ev.Attrs[ev3atime_sp].sData:=ms;
 ev.Attrs[ev3acommand].sData:=ev3mCommands[cRunTimed];
end;

function TEv3TachoMotor.RunTime(ms: variant): TEv3TachoMotor;
begin
 result:=self.runtime(ms,fspeed);
end;

function TEv3TachoMotor.RunDeg(deg: variant; sp: variant; sa: variant
  ): TEv3TachoMotor;
begin
 setHowStop(sa);
 result:=self.rundeg(deg,sp);
end;

function TEv3TachoMotor.RunDeg(deg: variant; sp: variant): TEv3TachoMotor;
begin
  result:=RunTurn(deg/360.0,sp);
end;

function TEv3TachoMotor.RunDeg(deg: variant): TEv3TachoMotor;
begin
  result:=RunTurn(deg/360.0);
end;

function TEv3TachoMotor.RunTurn(turn: variant; sp: variant; sa: variant
  ): TEv3TachoMotor;
begin
 setHowStop(sa);
 result:=RunTurn(turn,sp);
end;

function TEv3TachoMotor.RunTurn(turn: variant; sp: variant): TEv3TachoMotor;
begin
 result:=self;
 if fcount_per_rot=0 then fcount_per_rot:=ev.Attrs[ev3acount_per_rot].iData;
 setSpeed(sp);
// writeln('turn - ',Round(double(turn)*fcount_per_rot));
 ev.Attrs[ev3aposition_sp].iData:=Round(double(turn)*fcount_per_rot);
 ev.Attrs[ev3acommand].sData:=ev3mCommands[cRunToRelPos];
end;

function TEv3TachoMotor.RunTurn(turn: variant): TEv3TachoMotor;
begin
  result:=self;
  RunTurn(turn,fspeed);
end;

//--------------------------
procedure Done;
  var ev:TEv3Device;
 begin
  for ev in ev3DeviceFactory do begin
    case ev.Device of
     ev3tLTachoMotor,ev3tMTachoMotor: begin
//      writeln('+++');
//      writeln(strerror(fpgeterrno));
//      ev.Attrs[ev3acommand].sData:=ev3mCommands[cReset];
//      writeln(ev3mCommands[cReset]);
//      ev.Attrs[ev3acommand].hardPutData(ev3mCommands[cReset]);
//      writeln(strerror(fpgeterrno));
      ev.Attrs[ev3astop_action].sData:=ev3mStopActions[saCoast];
      ev.Attrs[ev3acommand].sData:=ev3mCommands[cStop];
//      writeln('----');
    //  readln;
     end;
    end;
  end;
 end;

initialization
 pushDone(@Done);
end.

